#include <glib.h>
#include <libipuz/libipuz.h>
#include <locale.h>


/* FIXME(tests): This one should actually fail. See Issue #29 */
#if 0
static void
test_clue_sets_duplicates (void)
{
  g_autoptr (IpuzPuzzle) puzzle = NULL;
  GError *error = NULL;
  gchar *path;

  setlocale(LC_ALL, "en_US.utf8");

  path = g_test_build_filename (G_TEST_DIST, "clue-sets-fail.ipuz", NULL);
  puzzle = ipuz_puzzle_new_from_file (path, &error);
  g_free (path);

  g_assert (puzzle == NULL);
}
#endif

static void
test_clue_sets_one_custom (void)
{
  g_autoptr (IpuzPuzzle) puzzle = NULL;
  GError *error = NULL;
  gchar *path;

  setlocale(LC_ALL, "en_US.utf8");

  path = g_test_build_filename (G_TEST_DIST, "clue-sets1.ipuz", NULL);
  puzzle = ipuz_puzzle_new_from_file (path, &error);
  g_free (path);

  if (error != NULL)
    {
      g_print ("Error: %s\n", error->message);
    }
  g_assert (puzzle != NULL);

  g_assert (ipuz_clues_get_n_clue_sets (IPUZ_CLUES (puzzle)) == 2);

}

static void
test_clue_sets_two_customs (void)
{
  g_autoptr (IpuzPuzzle) puzzle = NULL;
  GError *error = NULL;
  gchar *path;

  setlocale(LC_ALL, "en_US.utf8");

  path = g_test_build_filename (G_TEST_DIST, "clue-sets2.ipuz", NULL);
  puzzle = ipuz_puzzle_new_from_file (path, &error);
  g_free (path);

  if (error != NULL)
    {
      g_print ("Error: %s\n", error->message);
    }
  g_assert (puzzle != NULL);

  ipuz_crossword_print (IPUZ_CROSSWORD (puzzle));
}

static void
test_clue_sets_directions (void)
{
  g_autofree gchar *path = NULL;
  g_autofree gchar *path2 = NULL;
  g_autoptr (IpuzPuzzle) puzzle = NULL;
  g_autoptr (IpuzPuzzle) puzzle2 = NULL;
  g_autoptr (GError) error = NULL;

  setlocale(LC_ALL, "en_US.utf8");

  path = g_test_build_filename (G_TEST_DIST, "4962.ipuz", NULL);
  puzzle = ipuz_puzzle_new_from_file (path, &error);
  if (error != NULL)
    g_print ("Error: %s\n", error->message);
  g_assert (puzzle != NULL);

  /* After ACROSS is DOWN */
  g_assert_true (ipuz_clues_clue_set_next_direction (IPUZ_CLUES (puzzle),
                                                     IPUZ_CLUE_DIRECTION_ACROSS,
                                                     FALSE) == IPUZ_CLUE_DIRECTION_DOWN);

  /* Before DOWN is ACROSS */
  g_assert_true (ipuz_clues_clue_set_prev_direction (IPUZ_CLUES (puzzle),
                                                     IPUZ_CLUE_DIRECTION_DOWN,
                                                     FALSE) == IPUZ_CLUE_DIRECTION_ACROSS);

  /* Nothing after down... */
  g_assert_true (ipuz_clues_clue_set_next_direction (IPUZ_CLUES (puzzle),
                                                     IPUZ_CLUE_DIRECTION_DOWN,
                                                     FALSE) == IPUZ_CLUE_DIRECTION_NONE);

  /* ...unless we wrap */
  g_assert_true (ipuz_clues_clue_set_next_direction (IPUZ_CLUES (puzzle),
                                                     IPUZ_CLUE_DIRECTION_DOWN,
                                                     TRUE) == IPUZ_CLUE_DIRECTION_ACROSS);
  /* Nothing before across... */
  g_assert_true (ipuz_clues_clue_set_prev_direction (IPUZ_CLUES (puzzle),
                                                     IPUZ_CLUE_DIRECTION_ACROSS,
                                                     FALSE) == IPUZ_CLUE_DIRECTION_NONE);

  /* ...unless we wrap */
  g_assert_true (ipuz_clues_clue_set_next_direction (IPUZ_CLUES (puzzle),
                                                     IPUZ_CLUE_DIRECTION_ACROSS,
                                                     TRUE) == IPUZ_CLUE_DIRECTION_DOWN);

  path2 = g_test_build_filename (G_TEST_DIST, "clue-sets4.ipuz", NULL);
  puzzle2 = ipuz_puzzle_new_from_file (path2, &error);
  if (error != NULL)
    g_print ("Error: %s\n", error->message);
  g_assert (puzzle2 != NULL);

  /* clue-sets4 has Across/Hidden as its direction */
  /* This should go nowhere */
  g_assert_true (ipuz_clues_clue_set_next_direction (IPUZ_CLUES (puzzle2),
                                                     IPUZ_CLUE_DIRECTION_ACROSS,
                                                     FALSE) == IPUZ_CLUE_DIRECTION_NONE);
  
  /* This should go nowhere as well */
  g_assert_true (ipuz_clues_clue_set_prev_direction (IPUZ_CLUES (puzzle2),
                                                     IPUZ_CLUE_DIRECTION_ACROSS,
                                                     FALSE) == IPUZ_CLUE_DIRECTION_NONE);
  
  /* We wrap, so should go back to across */
  g_assert_true (ipuz_clues_clue_set_next_direction (IPUZ_CLUES (puzzle2),
                                                     IPUZ_CLUE_DIRECTION_ACROSS,
                                                     TRUE) == IPUZ_CLUE_DIRECTION_ACROSS);

  /* This as well */
  g_assert_true (ipuz_clues_clue_set_prev_direction (IPUZ_CLUES (puzzle2),
                                                     IPUZ_CLUE_DIRECTION_ACROSS,
                                                     TRUE) == IPUZ_CLUE_DIRECTION_ACROSS);
}

int
main (int   argc,
      char *argv[])
{
  g_test_init (&argc, &argv, NULL);

  g_test_add_func ("/clue-sets/one-custom", test_clue_sets_one_custom);
  g_test_add_func ("/clue-sets/two-customs", test_clue_sets_two_customs);
  g_test_add_func ("/clue-sets/test-directions", test_clue_sets_directions);

  return g_test_run ();
}

